/*****************************************************************************
project:  test of sht11 humidity and temperature sensor
author:   Werner Hoch <werner.ho@gmx.de>
file:     main.c
*****************************************************************************/

#include <avr/signal.h>          /* interrupt vectors */
#include <avr/interrupt.h>       /* sei(), cli(),... */
#include <stdlib.h>              /* itoa() for example */
#include <string.h>
#include "global.h"
#include "rs232.h"
#include "delay.h"
#include "sht11.h"

void
sendvalue(int16_t value) {
  uint8_t len,i;
  char b[7];
  if (value < 0) {
    value = -value;
    rs232_sendbyte('-');
  }
  itoa(value, b, 10);
  len = strlen(b);
  if (len < 3)
    rs232_sendbyte('0');
  if (len == 1) {
    rs232_sendbyte(',');
    rs232_sendbyte('0');
  }
  for (i=0; b[i];i++) {
    if (i == len - 2) { 
      rs232_sendbyte(',');
    }
    rs232_sendbyte(b[i]);
  }
}

int
main() {
  uint16_t sht11_value;
  uint16_t i;
  uint8_t noError = 1;

  rs232_init();

  /* sei();  */ /* enable global interrupts */

  /* sendvalue() test procedure */
  sendvalue(-5);
  rs232_sendbyte('\n');
  sendvalue(-20);
  rs232_sendbyte('\n');
  sendvalue(40);
  rs232_sendbyte('\n');
  sendvalue(3);
  rs232_sendbyte('\n');
  
  /*  sht11_humidity_scale() test procedure */
  for (i=100; i<4096; i++) {
    sendvalue(i);
    rs232_sendbyte('\t');  
    sht11_value = sht11_humidity_scale(i);
    sendvalue(sht11_value);
    rs232_sendbyte('\n');
  }
  
  delay(3);        /* wait at least 11ms after power-up */
  sht11_init();  
  sht11_reset();
  delay(3);        /* wait at least 11ms after reset */

  for (;;) {
    if (!noError) {
      sht11_reset();
      delay(3);
      rs232_sendbyte('r');
    }

    /* send temperature in hexadezimal numbers */
    noError=sht11_measure_start( sht11MeaT );
    if (!noError) {
      rs232_sendbyte('m');  /* send arbitrary letter to detect error location */
      continue;
    }
    noError = sht11_get_measure_value( &sht11_value );
    if (!noError) {
      rs232_sendbyte('n');  
      continue;
    }
    sht11_value -= 4000;

    /* send temperature in decimal numbers */
    rs232_sendbyte('T');
    rs232_sendbyte('\t');  
    sendvalue(sht11_value);

    /* get humidity value */
    noError=sht11_measure_start( sht11MeaRh );
    if (!noError) {
      rs232_sendbyte('o');  
      continue;
    }
    noError = sht11_get_measure_value( &sht11_value );
    if (!noError) {
      rs232_sendbyte('p');  
      continue;
    }

    /* scale the humidity value */
    sht11_value = sht11_humidity_scale(sht11_value);
    
    /* send the humidity as a dezimal number */
    rs232_sendbyte('\t');
    rs232_sendbyte('H');
    rs232_sendbyte('\t');  
    sendvalue(sht11_value);
    rs232_sendbyte('\n');
    delay(250);  /* about 1 second */
    delay(250);  /* about 1 second */
  }
}
